import 'dart:async';
import 'dart:collection';
import 'dart:developer';
import 'dart:io';
import 'package:easynetflutter/easynet/log/Logger.dart';
import 'package:easynetflutter/easynet/model/MessagePush.dart';
import 'package:easynetflutter/easynet/model/NotificationPush.dart';
import 'package:easynetflutter/easynet/util/EasyPlatfom.dart';
import 'package:easynetflutter/easynet/widget/LifecycleWatcherState.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/material.dart';

class PushFCMHandler {
  Logger logger;
  static PushFCMHandler _instance;
  final FirebaseMessaging _firebaseMessaging = FirebaseMessaging();

  //String _message = '';
  String _token;

  PushFCMHandler._internal() {
    logger = Logger(className: this.toString());
    if (EasyPlatfom.isIOS()) {
      _firebaseMessaging
          .requestNotificationPermissions(IosNotificationSettings());
    }
    register();
    getMessage();
  }

  List<LifecycleWatcherState> listObserverView = List<LifecycleWatcherState>();

  static PushFCMHandler getInstance() {
    if (_instance == null) {
      _instance = PushFCMHandler._internal();
    }
    return _instance;
  }

  void register() async {
    // _firebaseMessaging.getToken().then((token) => print(token));
    _token = await _firebaseMessaging.getToken();
    logger.w("Token PFCM: $_token");
    onCreateToken(_token);
    //SessionData.saveToken(_token);
  }

  FirebaseMessaging getFirebaseMessage() {
    return _firebaseMessaging;
  }

  Future<dynamic> backgroundMessageHandler(Map<String, dynamic> message) async {
    logger.w("backgroundMessageHandler ${message}");
    if (message.containsKey('data')) {
      await onBackgroundMessage(message);
      return Future<void>.value();
    }
  }

  void getMessage() {
    logger.w("Configurando FirebaseMessaging");
    _firebaseMessaging.configure(
        onBackgroundMessage: Platform.isIOS ? null : backgroundMessageHandler,
        onMessage: (Map<String, dynamic> message) async {
          onMessage(message);
        },
        onResume: (Map<String, dynamic> message) async {
          onResume(message);
        },
        onLaunch: (Map<String, dynamic> message) async {
          onLaunch(message);
        });
  }

  PushFCMHandler addObserverView(LifecycleWatcherState observerView) {
    listObserverView.add(observerView);
    return _instance;
  }

  void onBackgroundMessage(Map<String, dynamic> message) async {
    logger.w('on message $message');
    if (listObserverView.length > 0) {
      logger.w(message["notification"]["title"]);
      String title = message["notification"]["title"];
      String body = message["notification"]["body"];
      MessagePush messagePush = MessagePush(title, body);
      LinkedHashMap<dynamic, dynamic> dataPush = message["data"];
      NotificationPush notificationPush =
          NotificationPush(messagePush, dataPush);
      //var notificationPush = NotificationPush.fromJson(message);
      for (var observerView in listObserverView) {
        observerView.onMessagePush(notificationPush);
      }
    }
  }

  void onMessage(Map<String, dynamic> message) async {
    logger.w('on message $message');
    if (listObserverView.length > 0) {
      logger.w(message["notification"]["title"]);
      String title = message["notification"]["title"];
      String body = message["notification"]["body"];
      MessagePush messagePush = MessagePush(title, body);
      LinkedHashMap<dynamic, dynamic> dataPush = message["data"];
      NotificationPush notificationPush =
          NotificationPush(messagePush, dataPush);
      //var notificationPush = NotificationPush.fromJson(message);
      for (var observerView in listObserverView) {
        observerView.onMessagePush(notificationPush);
      }
    }
  }

  void onCreateToken(String token) {
    logger.w("onCreateToken $token");
    if (listObserverView.length > 0) {
      for (var observerView in listObserverView) {
        observerView.onCreateToken(token);
      }
    }
  }

  void onResume(Map<String, dynamic> message) {
    logger.w('on resume $message');
    if (listObserverView.length > 0) {
      logger.w(message["notification"]["title"]);
      String title = message["notification"]["title"];
      String body = message["notification"]["body"];
      MessagePush messagePush = MessagePush(title, body);
      LinkedHashMap<dynamic, dynamic> dataPush = message["data"];
      NotificationPush notificationPush =
          NotificationPush(messagePush, dataPush);
      for (var observerView in listObserverView) {
        observerView.onResumePush(notificationPush);
      }
    }
  }

  void onLaunch(Map<String, dynamic> message) {
    logger.w('on launch $message');
    if (listObserverView.length > 0) {
      String title = message["notification"]["title"];
      String body = message["notification"]["body"];
      MessagePush messagePush = MessagePush(title, body);
      LinkedHashMap<dynamic, dynamic> dataPush = message["data"];
      NotificationPush notificationPush =
          NotificationPush(messagePush, dataPush);
      for (var observerView in listObserverView) {
        observerView.onLaunchPush(notificationPush);
      }
    }
  }
}
